@@ -11,7 +11,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: environ.c 7986 2016-12-10 11:44:07Z jccleaver $";
+static char rcsid[] = "$Id: environ.c 7986M 2019-07-23 14:46:51Z (local) $";
 
 #include <ctype.h>
 #include <string.h>
@@ -166,13 +166,15 @@ const static struct {
 
 char *xgetenv(const char *name)
 {
-	char *result, *newstr;
+	char *result;
+	SBUF_DEFINE(newstr);
 	int i;
 
 	result = getenv(name);
 	if ((result == NULL) && (strcmp(name, "MACHINE") == 0) && xgetenv("MACHINEDOTS")) {
 		/* If MACHINE is undefined, but MACHINEDOTS is there, create MACHINE  */
-		char *oneenv, *p;
+		SBUF_DEFINE(oneenv);
+		char *p;
 		
 #ifdef HAVE_SETENV
 		oneenv = strdup(xgetenv("MACHINEDOTS"));
@@ -180,8 +182,8 @@ char *xgetenv(const char *name)
 		setenv(name, oneenv, 1);
 		xfree(oneenv);
 #else
-		oneenv = (char *)malloc(10 + strlen(xgetenv("MACHINEDOTS")));
-		sprintf(oneenv, "%s=%s", name, xgetenv("MACHINEDOTS"));
+		SBUF_MALLOC(10 + strlen(xgetenv("MACHINEDOTS")));
+		snprintf(oneenv, oneenv_buflen, "%s=%s", name, xgetenv("MACHINEDOTS"));
 		p = oneenv; while ((p = strchr(p, '.')) != NULL) *p = ',';
 		putenv(oneenv);
 #endif
@@ -203,8 +205,8 @@ char *xgetenv(const char *name)
 #ifdef HAVE_SETENV
 		setenv(name, result, 1);
 #else
-		newstr = malloc(strlen(name) + strlen(result) + 2);
-		sprintf(newstr, "%s=%s", name, result);
+		SBUF_MALLOC(newstr, strlen(name) + strlen(result) + 2); 
+		snprintf(newstr, newstr_buflen, "%s=%s", name, result);
 		putenv(newstr);
 #endif
 		/*
@@ -238,7 +240,8 @@ void loadenv(char *envfile, char *area)
 {
 	FILE *fd;
 	strbuffer_t *inbuf;
-	char *p, *marker, *oneenv;
+	char *p, *marker;
+	SBUF_DEFINE(oneenv);
 
 	MEMDEFINE(l);
 	inbuf = newstrbuffer(0);
@@ -271,10 +274,16 @@ void loadenv(char *envfile, char *area)
 			if (*marker == '/') {
 				if (area) {
 					*marker = '\0';
-					if (strcasecmp(p, area) == 0) oneenv = strdup(expand_env(marker+1));
+					if (strcasecmp(p, area) == 0) {
+						oneenv = strdup(expand_env(marker+1));
+						oneenv_buflen = strlen(oneenv)+1;
+					}
 				}
 			}
-			else oneenv = strdup(expand_env(p));
+			else {
+				oneenv = strdup(expand_env(p));
+				oneenv_buflen = strlen(oneenv)+1;
+			}
 
 			if (oneenv) {
 				p = strchr(oneenv, '=');
@@ -293,14 +302,16 @@ void loadenv(char *envfile, char *area)
 
 					oldval = getenv(oneenv);
 					if (oldval) {
-						char *combinedenv = (char *)malloc(strlen(oneenv) + strlen(oldval) + strlen(addstring) + 2);
-						sprintf(combinedenv, "%s=%s%s", oneenv, oldval, (addstring));
+						SBUF_DEFINE(combinedenv);
+
+						SBUF_MALLOC(combinedenv, strlen(oneenv) + strlen(oldval) + strlen(addstring) + 2);
+						snprintf(combinedenv, combinedenv_buflen, "%s=%s%s", oneenv, oldval, (addstring));
 						xfree(oneenv);
 						oneenv = combinedenv;
 					}
 					else {
 						/* oneenv is now VARxxVALUE, so fix it to be a normal env. variable format */
-						strcat(oneenv, "=");
+						strncat(oneenv, "=", oneenv_buflen-strlen(oneenv));
 						memmove(oneenv+strlen(oneenv), addstring, strlen(addstring) + 1);
 					}
 				}
@@ -324,8 +335,9 @@ char *getenv_default(char *envname, char *envdefault, char **buf)
 
 	val = getenv(envname);	/* Don't use xgetenv() here! */
 	if (!val) {
-		val = (char *)malloc(strlen(envname) + strlen(envdefault) + 2);
-		sprintf(val, "%s=%s", envname, envdefault);
+		unsigned int val_buflen = strlen(envname) + strlen(envdefault) + 2;
+		val = (char *)malloc(val_buflen);
+		snprintf(val, val_buflen, "%s=%s", envname, envdefault);
 		putenv(val);
 		/* Don't free the string - it must be kept for the environment to work */
 		val = xgetenv(envname);	/* OK to use xgetenv here */
@@ -372,7 +384,7 @@ char *expand_env(char *s)
 			myxp->resultlen += strlen(bot) + 4096;
 			myxp->result = (char *)realloc(myxp->result, myxp->resultlen);
 		}
-		strcat(myxp->result, bot);
+		strncat(myxp->result, bot, (myxp->resultlen - strlen(myxp->result)));
 
 		if (tstart) {
 			tstart++;
@@ -405,7 +417,7 @@ char *expand_env(char *s)
 					myxp->resultlen += strlen(envval) + 4096;
 					myxp->result = (char *)realloc(myxp->result, myxp->resultlen);
 				}
-				strcat(myxp->result, envval);
+				strncat(myxp->result, envval, (myxp->resultlen - strlen(myxp->result)));
 			}
 		}
 		else {
