@@ -11,7 +11,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: htmllog.c 7861 2016-01-20 18:50:38Z jccleaver $";
+static char rcsid[] = "$Id: htmllog.c 7861M 2019-07-23 14:46:51Z (local) $";
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -82,8 +82,9 @@ static void hostsvc_setup(void)
 	getenv_default("XYMONPAGEROWFONT", "SIZE=+1 COLOR=\"#FFFFCC\" FACE=\"Tahoma, Arial, Helvetica\"", &rowfont);
 	getenv_default("XYMONWEB", "/xymon", NULL);
 	{
-		char *dbuf = malloc(strlen(xgetenv("XYMONWEB")) + 6);
-		sprintf(dbuf, "%s/gifs", xgetenv("XYMONWEB"));
+		SBUF_DEFINE(dbuf);
+		SBUF_MALLOC(dbuf, strlen(xgetenv("XYMONWEB")) + 6);
+		snprintf(dbuf, dbuf_buflen, "%s/gifs", xgetenv("XYMONWEB"));
 		getenv_default("XYMONSKIN", dbuf, NULL);
 		xfree(dbuf);
 	}
@@ -94,14 +95,16 @@ static void hostsvc_setup(void)
 
 static void historybutton(char *cgibinurl, char *hostname, char *service, char *ip, char *displayname, char *btntxt, FILE *output) 
 {
-	char *tmp1;
-	char *tmp2 = (char *)malloc(strlen(service)+3);
+	SBUF_DEFINE(tmp1);
+	SBUF_DEFINE(tmp2)
+		
+	SBUF_MALLOC(tmp2, strlen(service)+3);
 
 	getenv_default("NONHISTS", "info,trends", NULL);
-	tmp1 =  (char *)malloc(strlen(xgetenv("NONHISTS"))+3);
+	SBUF_MALLOC(tmp1, strlen(xgetenv("NONHISTS"))+3);
 
-	sprintf(tmp1, ",%s,", xgetenv("NONHISTS"));
-	sprintf(tmp2, ",%s,", service);
+	snprintf(tmp1, tmp1_buflen, ",%s,", xgetenv("NONHISTS"));
+	snprintf(tmp2, tmp2_buflen, ",%s,", service);
 	if (strstr(tmp1, tmp2) == NULL) {
 		fprintf(output, "<BR><BR><CENTER><FORM ACTION=\"%s/history.sh\">", cgibinurl);
 		fprintf(output, "<INPUT TYPE=SUBMIT VALUE=\"%s\">", htmlquoted(btntxt));
@@ -182,7 +185,7 @@ void generate_html_log(char *hostname, char *displayname, char *service, char *i
 	xymonrrd_t *rrd = NULL;
 	xymongraph_t *graph = NULL;
 	char *tplfile = "hostsvc";
-	char *graphs;
+	SBUF_DEFINE(graphs);
 	char *graphsenv;
 	char *graphsptr;
 	time_t now = getcurrenttime(NULL);
@@ -207,7 +210,7 @@ void generate_html_log(char *hostname, char *displayname, char *service, char *i
 		int formfile;
 		char formfn[PATH_MAX];
 
-		sprintf(formfn, "%s/web/trends_form", xgetenv("XYMONHOME"));
+		snprintf(formfn, sizeof(formfn), "%s/web/trends_form", xgetenv("XYMONHOME"));
 		formfile = open(formfn, O_RDONLY);
 
 		if (formfile >= 0) {
@@ -231,7 +234,7 @@ void generate_html_log(char *hostname, char *displayname, char *service, char *i
 		int formfile;
 		char formfn[PATH_MAX];
 
-		sprintf(formfn, "%s/web/critack_form", xgetenv("XYMONHOME"));
+		snprintf(formfn, sizeof(formfn), "%s/web/critack_form", xgetenv("XYMONHOME"));
 		formfile = open(formfn, O_RDONLY);
 
 		if (formfile >= 0) {
@@ -433,7 +436,8 @@ void generate_html_log(char *hostname, char *displayname, char *service, char *i
 			linecount=atoi(lcstr+15);
 		}
 		else {
-			char *p, *multikey;
+			SBUF_DEFINE(multikey);
+			char *p;
 
 			/* quotas, snapshot and TblSpace are generated by hobbit-perl-client */
 			if (multigraphs == NULL) multigraphs = ",disk,inode,qtree,quotas,snapshot,TblSpace,if_load,";
@@ -448,8 +452,8 @@ void generate_html_log(char *hostname, char *displayname, char *service, char *i
 			 * RRD database files are present matching this service - but that is way too
 			 * much overhead for something that might be called on every status logged.
 			 */
-			multikey = (char *)malloc(strlen(service) + 3);
-			sprintf(multikey, ",%s,", service);
+			SBUF_MALLOC(multikey, strlen(service) + 3);
+			snprintf(multikey, multikey_buflen, ",%s,", service);
 			if (strstr(multigraphs, multikey)) {
 				/* The "disk" report from the NetWare client puts a "warning light" on all entries */
 				int netwarediskreport = (strstr(firstline, "NetWare Volumes") != NULL);
@@ -501,8 +505,8 @@ void generate_html_log(char *hostname, char *displayname, char *service, char *i
 			fprintf(output, "<a name=\"begingraph\">&nbsp;</a>\n");
 
 			/* Get the GRAPHS_* environment setting */
-			graphs = (char *)malloc(7 + strlen(service) + 1);
-			sprintf(graphs, "GRAPHS_%s", service);
+			SBUF_MALLOC(graphs, 7 + strlen(service) + 1);
+			snprintf(graphs, graphs_buflen, "GRAPHS_%s", service);
 			graphsenv=getenv(graphs);
 			if (graphsenv) {
 				fprintf(output, "<!-- GRAPHS_%s: %s -->\n", service, graphsenv);
@@ -553,7 +557,7 @@ char *alttag(char *columnname, int color, int acked, int propagate, char *age)
 
 static char *nameandcomment(void *host, char *hostname, int usetooltip)
 {
-	static char *result = NULL;
+	STATIC_SBUF_DEFINE(result);
 	char *cmt, *disp, *hname;
 
 	if (result) xfree(result);
@@ -576,12 +580,12 @@ static char *nameandcomment(void *host, char *hostname, int usetooltip)
 	if (cmt) {
 		if (usetooltip) {
 			/* Thanks to Marco Schoemaker for suggesting the use of <span title...> */
-			result = (char *)malloc(strlen(disp) + strlen(cmt) + 30);
-			sprintf(result, "<span title=\"%s\">%s</span>", cmt, disp);
+			SBUF_MALLOC(result, strlen(disp) + strlen(cmt) + 30);
+			snprintf(result, result_buflen, "<span title=\"%s\">%s</span>", cmt, disp);
 		}
 		else {
-			result = (char *)malloc(strlen(disp) + strlen(cmt) + 4);
-			sprintf(result, "%s (%s)", disp, cmt);
+			SBUF_MALLOC(result, strlen(disp) + strlen(cmt) + 4);
+			snprintf(result, result_buflen, "%s (%s)", disp, cmt);
 		}
 		return result;
 	}
@@ -600,7 +604,7 @@ static char *urldoclink(const char *docurl, const char *hostname)
 	static char linkurl[PATH_MAX];
 
 	if (docurl) {
-		sprintf(linkurl, docurl, hostname);
+		snprintf(linkurl, sizeof(linkurl), docurl, hostname);
 	}
 	else {
 		linkurl[0] = '\0';
@@ -642,6 +646,9 @@ char *hostnamehtml(char *hostname, char *defaultlink, int usetooltip)
 	 *
 	 * else just put the hostname there.
 	 */
+
+	#pragma GCC diagnostic push
+	#pragma GCC diagnostic ignored "-Wformat-truncation"
 	if (documentationurl) {
 		snprintf(result, sizeof(result), "<A HREF=\"%s\" %s><FONT %s>%s</FONT></A>",
 			urldoclink(documentationurl, hostname),
@@ -661,6 +668,7 @@ char *hostnamehtml(char *hostname, char *defaultlink, int usetooltip)
 		snprintf(result, sizeof(result), "<FONT %s>%s</FONT>",
 			xgetenv("XYMONPAGEROWFONT"), nameandcomment(hinfo, hostname, usetooltip));
 	}
+	#pragma GCC diagnostic pop
 
 	return result;
 }
