@@ -10,7 +10,7 @@
 /*                                                                            */
 /*----------------------------------------------------------------------------*/
 
-static char rcsid[] = "$Id: httptest.c 7978 2016-10-31 18:58:13Z jccleaver $";
+static char rcsid[] = "$Id: httptest.c 7978M 2019-07-23 14:46:51Z (local) $";
 
 #include <sys/types.h>
 #include <limits.h>
@@ -395,7 +395,7 @@ void add_http_test(testitem_t *t)
 		{
 			FILE *contentfd;
 			char contentfn[PATH_MAX];
-			sprintf(contentfn, "%s/content/%s.substring", xgetenv("XYMONHOME"), commafy(t->host->hostname));
+			snprintf(contentfn, sizeof(contentfn), "%s/content/%s.substring", xgetenv("XYMONHOME"), commafy(t->host->hostname));
 			contentfd = fopen(contentfn, "r");
 			if (contentfd) {
 				char l[MAX_LINE_LEN];
@@ -529,7 +529,7 @@ void add_http_test(testitem_t *t)
 	if ((httptest->weburl.desturl->port != 80) && (httptest->weburl.desturl->port != 443)) {
 		char hostporthdr[20];
 
-		sprintf(hostporthdr, ":%d", httptest->weburl.desturl->port);
+		snprintf(hostporthdr, sizeof(hostporthdr), ":%d", httptest->weburl.desturl->port);
 		addtobuffer(httprequest, hostporthdr);
 	}
 	addtobuffer(httprequest, "\r\n");
@@ -584,7 +584,7 @@ void add_http_test(testitem_t *t)
 			addtobuffer(httprequest, "application/x-www-form-urlencoded");
 		addtobuffer(httprequest, "\r\n");
 
-		sprintf(hdr, "Content-Length: %d\r\n", contlen);
+		snprintf(hdr, sizeof(hdr), "Content-Length: %d\r\n", contlen);
 		addtobuffer(httprequest, hdr);
 	}
 	{
@@ -595,10 +595,10 @@ void add_http_test(testitem_t *t)
 		if (hinfo) browser = xmh_item(hinfo, XMH_BROWSER);
 
 		if (browser) {
-			sprintf(useragent, "User-Agent: %s\r\n", browser);
+			snprintf(useragent, sizeof(useragent), "User-Agent: %s\r\n", browser);
 		}
 		else {
-			sprintf(useragent, "User-Agent: Xymon xymonnet/%s\r\n", VERSION);
+			snprintf(useragent, sizeof(useragent), "User-Agent: Xymon xymonnet/%s\r\n", VERSION);
 		}
 
 		addtobuffer(httprequest, useragent);
